package de.fraunhofer.iem.app.kpi.strategy

import de.fraunhofer.iem.app.kpi.dto.KPIHierarchyEdgeDto

class AggregationKPICalculationStrategy : de.fraunhofer.iem.app.kpi.strategy.KPICalculationStrategy {

    /**
     * This function calculates the aggregate sum of all given children.
     * If a child is empty it is removed from the calculation and its
     * corresponding edge weight is distributed evenly between the
     * remaining children.
     * The method returns the KPIs value as well as the updated
     * KPIHierarchyEdgeDtos with the actual used weight.
     */
    override fun calculateKPI(children: List<KPIHierarchyEdgeDto>): Pair<Int, List<KPIHierarchyEdgeDto>> {
        if (children.none { !it.to.isEmpty() } || children.isEmpty()) {
            throw Exception("KPI aggregation of empty children can't be calculated.")
        }

        val emptyChildren: List<KPIHierarchyEdgeDto> = children.filter { it.to.isEmpty() }

        val notEmptyChildren = children.toMutableList()
        notEmptyChildren.removeAll(emptyChildren)

        val additionalWeightPerElement: Double = if (notEmptyChildren.isNotEmpty()) {
            val unusedWeight = emptyChildren.sumOf { it.plannedWeight }
            unusedWeight / notEmptyChildren.size
        } else {
            0.0
        }

        val weightedEdges: MutableList<KPIHierarchyEdgeDto> = emptyChildren
            .map {
                KPIHierarchyEdgeDto(
                    from = it.from,
                    to = it.to,
                    plannedWeight = it.plannedWeight,
                    actualWeight = 0.0
                )
            }.toMutableList()

        var value = 0

        notEmptyChildren.forEach { child ->
            val actualWeight = (child.plannedWeight + additionalWeightPerElement)
            weightedEdges.add(
                KPIHierarchyEdgeDto(
                    from = child.from,
                    to = child.to,
                    plannedWeight = child.plannedWeight,
                    actualWeight = actualWeight
                )
            )
            value += (child.to.getValue().toFloat() * actualWeight).toInt()
        }

        return Pair(value, weightedEdges)
    }
}
